# Sage version information for Python scripts
# This file is auto-generated by the sage-update-version script, do not edit!
version = '10.5.beta0'
date = '2024-07-24'
banner = 'SageMath version 10.5.beta0, Release Date: 2024-07-24'
